###############################################################################
# stephen_cpu: 
#    - demonstration code using the sample p95 and avg data
#      to train an svm model and use it to predict classifications.
#    - svm model is also tuned to improve the classification process by adjusting 
#      the cost and gamma
#    - confusion and percentage are provided to review prediction results
#    - visualations of training and miss-classidied data are also provided to "SEE"
#      the features in the data the svm model is using
#
#  REQUIRED SOFTWARE: R 2.11.1
#
#  RECOMMEND USE:
#    The code is intended for demonstration pruposes.  I recommed the code
#       is stepped through a few lines a time, from ### COMMENT to the next
#       blank line.
# 
# Author: Stephen OConnell
###############################################################################
## CLEAN SWEEP
## rm(list = ls())

## SOME SUPPORTING PACKAGES
require(gplots)
require(lattice)
require(reshape)
require(Cairo)

##----------------------------------------------------------------------
##  SET THE WORKING DIRECTORY
##----------------------------------------------------------------------
## YOU NEED TO SET THE WORKING DIRECTORY TO WHERE THE DATA FILES ARE LOCATED
dataDir  <- "E:/TEMP/201001/"

setwd(dataDir)
getwd()

##----------------------------------------------------------------------
##  READ IN THE TRAINING DATA
##----------------------------------------------------------------------
p95 <- dget("p95_matrix.Rdget")
avg <- dget("avg_matrix.Rdget")
lbl <- dget("labels_y.Rdget")

# FIX THE 'x' SAMPLES
lbl$Y[lbl$SAMPLE == 'SAMPLE_49'] <- '5'
lbl$Y[lbl$SAMPLE == 'SAMPLE_786'] <- '5'
lbl$Y[lbl$SAMPLE == 'SAMPLE_1250'] <- '3'

## CREATE AN INTEGER VECTOR OF Y FOR LATER PROCESSING 
y <- as.integer(lbl$Y)

##----------------------------------------------------------------------
##  RUN SVM
##----------------------------------------------------------------------
## LOAD THE e1071 package
require(e1071)

###  THE NUMBER OF EACH CLASS IN THE TRAINING DATA
summary(as.factor(lbl$Y))	

### COMBINE THE p95 AND avg MATRIX TO CREATE A COMPLETE SET OF FEATURES
comb_matrix <- cbind(p95, avg)

###  CREATE THE TRAINED SVM MODEL AND CREATE A PREDICTION USING THE MODEL
model <- svm(comb_matrix, y, type='C')
pred <- predict(model, comb_matrix)

### CONFUSION MATRIX
table(pred,y)

### PERCENTAGE OF PROPERLY CLASSIED SAMPLES
df_y <- table(y)
result <- vector()
for (i in 1:6) {result[i] <- table(pred,y)[i,i]/df_y[i]}
t(round(result*100,1))


###---------------------------------------
###  TUNING - USING tune.svm TO FIND THE 'BEST' VALUES FOR COST AND GAMMA
###---------------------------------------
###  DATA NEEDS TO BE IN ONE DATA FRAME FOR THE tune.svm FUNCTION
DF_comb <- as.data.frame(comb_matrix)
DF_comb <- cbind(class=lbl$Y, DF_comb)
DF_comb$class <- as.factor(DF_comb$class)

### SHOW THE STRUCTURE OF DATA FRAME
str(DF_comb)

## RUNS A VERY VERY LONG TIME 
##tune_it <- tune.svm(class~., data = DF_comb, gamma = seq(from=.8, to=1.2, by=.1), cost = seq(from=1, to=5, by=1))

## LOAD A TUNED OBJECT - SAVED TIME IN RUNNING THE TUNE - USED JUST p95 DATA
load("tune_c.Rdata")

## SHOW THE RESULTS OF TUNING
print(tune_c_model)

## SHOW THE SUMMARY OF TUNING - INCLUDES RESULTS OF ALL ATTEMPTS
summary(tune_c_model)

## PLOT THE GRID RESULTS
plot(tune_c_model)



## BEST FIT TUNED RESULTS AND PREDICTION (BASED ON p95, DUE TO AMOUNT OF TIME REQUIRED TO RUN COMBINED MATRIX)
c_model <- svm(comb_matrix, lbl$Y, type="C-classification", gamma=.9, cost=3.5)
c_new <- predict(c_model, comb_matrix)

## CONFUSION MATRIX
table(c_new,y)

### PERCENTAGE OF PROPERLY CLASSIED SAMPLES
for (i in 1:6) {result[i] <- table(c_new,y)[i,i]/df_y[i]}
t(round(result*100,1))


###--------------------------
### SHOW THE MISCLASSIFIED SAMPLES - CAN BE USED TO REVIEW SPECIFIC SAMPLES
###   FROM THRE TRAINING DATASET - OR TEST/REAL DATA
###--------------------------
## SAMPLE NUMBER AND TRAINING CLASSIFICATION
c_new[c_new != y]

## WHAT THEY WERE 'MISCLASSIFIED' AS BASED ON THE SVM MODEL
lbl$Y[c_new != y]

## MISCLASSIED SAMPLE NAMES - INCLUDED IN THE FOLLOWING CHARTS
lbl$SAMPLE[c_new != y]



###--------------------------
### VISUAL - DATA SAMPLES BY CLASS - HELPS TO EXAMINE THE TRAINING (OR TEST DATA) 
###--------------------------

### JUST USING p95 MATRIX TO IMPROVE THE VISUALATION
###  CREATE A DATAFRAME STRUCTURED FOR THE VISUALATION
DF_comb <- as.data.frame(p95)
DF_comb <- cbind(class=lbl$Y, DF_comb)
DF_comb$class <- as.factor(DF_comb$class)

DF_comb <- cbind(sample=lbl$SAMPLE, DF_comb)
DF_comb$sample <- as.character(DF_comb$sample)
DF_comb$class <- as.character(DF_comb$class)

###  melt IS USED TO REDUCE (MELT) THE MATRIX TO A ROW WITH ONE FEATURE(VALUE) 
###  AND KEY DATA (SAMPLE AND CLASS) TO ID THE FEATURE
###  THE RESULTING DATA STRUCTURE LENDS ITSELF TO EASIER PLOTTING
M_comb <- melt(DF_comb, id=c("sample", "class"))

### SHOW THE MELTED STRUCUTRE
str(M_comb)


###----
### The following code can be used to generate visuals for each class
###   (NOT A VERY GOOD CODING EXAMPLE, JUST A QUICK HACK, TO PRODUCE VISUALS)
###
### The visuals can be used to help tune the training data to remove samples
###   that are possibly contributing to the misclassification
###
### Run the subset and chart_tile lines of code and then skip
### to the XYPLOT statement to produce the chart
###----

## SHOW THE 2 OR BROKEN 
ss_M_comb <- subset(M_comb, class == '2')
chart_title <- 'BROKEN'
### --- SKIP TO XYPLOT ---

## SHOW THE 5 OR FIXED
ss_M_comb <- subset(M_comb, class == '5')
chart_title <- "FIXED"
### --- SKIP TO XYPLOT ---

## SHOW THE 6 OR DECLINING
ss_M_comb <- subset(M_comb, class == '6')
chart_title <- "DECLINING"
### --- SKIP TO XYPLOT ---

## SHOW THE 4 OR MONITOR (PRUNED TO FIRST 60, there are too many to make a reasonable chart)
## CREATES A LIST OF THE FIRST 60 SAMPLES
samples_pruned <- subset(DF_comb, class == '4', select=c('sample'))[1:60,]
## SUBSET ONLY THE SAMPLES ON THE PRUNED LIST OF SAMPLES
ss_M_comb <- subset(M_comb, class == '4' & sample %in% samples_pruned)
chart_title <- "MONITOR"
### --- SKIP TO XYPLOT ---

## SHOW THE 1 OR LOW (PRUNED TO FIRST 60, there are too many to make a reasonable chart)
## CREATES A LIST OF THE FIRST 60 SAMPLES
samples_pruned <- subset(DF_comb, class == '1', select=c('sample'))[1:60,]
## SUBSET ONLY THE SAMPLES ON THE PRUNED LIST OF SAMPLES
ss_M_comb <- subset(M_comb, class == '1' & sample %in% samples_pruned)
chart_title <- "LOW"
### --- SKIP TO XYPLOT ---

## SHOW THE 3 OR VARIABLE (PRUNED TO FIRST 60, there are too many to make a reasonable chart)
## CREATES A LIST OF THE FIRST 60 SAMPLES
samples_pruned <- subset(DF_comb, class == '3', select=c('sample'))[1:60,]
## SUBSET ONLY THE SAMPLES ON THE PRUNED LIST OF SAMPLES
ss_M_comb <- subset(M_comb, class == '3' & sample %in% samples_pruned)
chart_title <- "VARIABLE"
### --- SKIP TO XYPLOT ---

### SHOW THE MISLASSIFIED SAMPLES
### 
ss_M_comb <- subset(M_comb, sample %in% lbl$SAMPLE[c_new != y])
chart_title <- "MISCLASSIFIED"

	
### --- XYPLOT ----
### RUN THE FOLLOWING AFTER RUNNING subset and chart title

xyplot(value ~ variable | factor(sample), data=ss_M_comb,
		main=paste(chart_title),
		type='l',
		table=TRUE,
		ylim=c(0,100),
		par.strip.text=list(cex=.7),
		scales = list(x=list(draw=FALSE))
)
 